#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>

#include "config.h"
#include "imapfilter.h"
#include "data.h"


extern int sockpri;

static unsigned int tag = 0xF00000;	/* Every IMAP command is prefixed
					   with a unique [:alnum:] string. */

/*
 * Send to server data; a command.
 */
unsigned int send_command(int *sock, char *cmd)
{
#ifdef DEBUG
    printf("debug: sending command: %s", cmd);
#endif

    socket_write(sock, cmd);

    return tag++;
}


#ifdef DEBUG
/*
 * IMAP NOOP: does nothing always succeeds.
 */
int imap_noop(int *sock)
{
    char cmd[SMALL_CMD];

    verbose("Client request: NOOP\n");

    snprintf(cmd, SMALL_CMD, "%X NOOP\r\n", tag);

    return send_command(sock, cmd);
}

#endif


/*
 * IMAP CAPABILITY: requests listing of capabilities that the server supports.
 */
int imap_capability(int *sock)
{
    char cmd[SMALL_CMD];

    verbose("Client request: CAPABILITY\n");

    snprintf(cmd, SMALL_CMD, "%X CAPABILITY\r\n", tag);

    return send_command(sock, cmd);
}


/*
 * IMAP NAMESPACE: discovers the prefix and delimeter of namespaces used by
 * the server for mailboxes (RFC 2342).
 */
int imap_namespace(int *sock)
{
    char cmd[SMALL_CMD];

    verbose("Client request: NAMESPACE\n");

    snprintf(cmd, SMALL_CMD, "%X NAMESPACE\r\n", tag);

    return send_command(sock, cmd);
}


/*
 * IMAP LOGOUT: informs server that client is done.
 */
int imap_logout(int *sock)
{
    char cmd[SMALL_CMD];

    verbose("Client request: LOGOUT\n");

    snprintf(cmd, SMALL_CMD, "%X LOGOUT\r\n", tag);

    return send_command(sock, cmd);
}


/*
 * IMAP LOGIN: identifies client to server.
 */
int imap_login(int *sock, char *user, char *pass)
{
    int r;
    char *cmd;

    cmd = (char *)smalloc(MEDIUM_CMD);

    verbose("Client request: LOGIN\n");

    snprintf(cmd, MEDIUM_CMD, "%X LOGIN \"%s\" \"%s\"\r\n", tag, user,
	     pass);

    r = send_command(sock, cmd);

    sfree(cmd);

    return r;
}


/*
 * IMAP LIST: returns a subset of names from the complete set of names
 * available to the client.
 *
int imap_list(int *sock, char *refer, char *mbox)
{
    int r;
    char cmd[BIG_CMD];

    verbose("Client request: LIST\n");

    snprintf(cmd, MEDIUM_CMD, "%X LIST \"%s\" \"%s\"\r\n", tag, refer,
	     mbox);

    r = send_command(sock, cmd);

    return r;
}*/


/*
 * IMAP SUBSCRIBE: adds the specified mailbox name to the server's
 * set of "active" or "subscribed" mailboxes.
 */
int imap_subscribe(int *sock, char *mbox)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: SUBSCRIBE\n");

    snprintf(cmd, MEDIUM_CMD, "%X SUBSCRIBE \"%s\"\r\n", tag, mbox);

    return send_command(sock, cmd);
}


/*
 * IMAP EXAMINE: access a mailbox in READ-ONLY mode.
 */
int imap_examine(int *sock, char *mbox)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: EXAMINE %s\n", mbox);

    snprintf(cmd, MEDIUM_CMD, "%X EXAMINE \"%s\"\r\n", tag, mbox);

    return send_command(sock, cmd);
}


/*
 * IMAP SELECT: access a mailbox in READ-WRITE mode.
 */
int imap_select(int *sock, char *mbox)
{
    char cmd[SMALL_CMD];

    verbose("Client request: SELECT\n");

    snprintf(cmd, SMALL_CMD, "%X SELECT \"%s\"\r\n", tag, mbox);

    return send_command(sock, cmd);
}


/*
 * IMAP STATUS: requests status of the indicated mailbox.
 */
int imap_status(int *sock, char *mbox, char *items)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: STATUS\n");

    snprintf(cmd, MEDIUM_CMD, "%X STATUS \"%s\" (%s)\r\n", tag, mbox, items);

    return send_command(sock, cmd);
}


/*
 * IMAP CREATE: create mailbox.
 */
int imap_create(int *sock, char *mbox)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: CREATE\n");

    snprintf(cmd, MEDIUM_CMD, "%X CREATE \"%s\"\r\n", tag, mbox);

    return send_command(sock, cmd);
}


/*
 * IMAP SEARCH: searches the mailbox for messages that match certain criteria.
 */
int imap_search(int *sock, char *search)
{
    char cmd[BIG_CMD];

    verbose("Client request: SEARCH\n");

    snprintf(cmd, BIG_CMD, "%X SEARCH %s\r\n", tag, search);

    return send_command(sock, cmd);
}


/*
 * IMAP FETCH: retrieves data associated with a message.
 */
int imap_fetch(int *sock, char *mesg, char *items)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: FETCH\n");

    snprintf(cmd, MEDIUM_CMD,
	     "%X FETCH %s %s\r\n", tag, mesg, items);

    return send_command(sock, cmd);
}


/*
 * IMAP STORE: alters data associated with a message.
 */
int imap_store(int *sock, char *mesg, unsigned int mode, char *flags)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: STORE\n");

    snprintf(cmd, MEDIUM_CMD, "%X STORE %s %sFLAGS.SILENT (%s)\r\n", tag, mesg,
	     (mode == STORE_FLAG_REPLACE ? "" :
	      mode == STORE_FLAG_ADD ? "+" : "-"), flags);

    return send_command(sock, cmd);
}


/*
 * IMAP COPY: copy messages to mailbox.
 */
int imap_copy(int *sock, char *mesg, char *mbox)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: COPY\n");

    snprintf(cmd, MEDIUM_CMD, "%X COPY %s \"%s\"\r\n", tag, mesg, mbox);

    return send_command(sock, cmd);
}


/*
 * IMAP APPEND: append message to the end of a mailbox.
 */
int imap_append(int *sock, char *mbox, unsigned int size)
{
    char cmd[MEDIUM_CMD];

    verbose("Client request: APPEND\n");

    snprintf(cmd, MEDIUM_CMD, "%X APPEND \"%s\" {%d}\r\n", tag, mbox, size);

    return send_command(sock, cmd);
}



/*
 * IMAP CLOSE: delete messages and return to authenticated state.
 */
int imap_close(int *sock)
{
    char cmd[SMALL_CMD];

    verbose("Client request: CLOSE\n");

    snprintf(cmd, SMALL_CMD, "%X CLOSE\r\n", tag);

    return send_command(sock, cmd);
}


/*
 * IMAP EXPUNGE: permanently removes any messages with the \Deleted flag set.
 */
int imap_expunge(int *sock)
{
    char cmd[SMALL_CMD];

    verbose("Client request: EXPUNGE\n");

    snprintf(cmd, SMALL_CMD, "%X EXPUNGE\r\n", tag);

    return send_command(sock, cmd);
}
